use crate::configuration::VcsType;
use crate::register_origin::models::OriginRequest;
use crate::Settings;

pub fn init(settings: &Settings) -> RegisterOriginHandler {
    let origin_gateway = OriginGateway::new(settings);
    RegisterOriginHandler {
        settings,
        origin_gateway,
    }
}

pub struct RegisterOriginHandler<'a> {
    settings: &'a Settings,
    origin_gateway: OriginGateway<'a>,
}

impl<'a> RegisterOriginHandler<'a> {
    pub fn execute(&self, request: OriginRequest) {
        self.origin_gateway.upsert_origin(
            request.name,
            request.owner_name,
            request.repo_name,
            request.vcs_type,
        );
    }
}

pub struct OriginGateway<'a> {
    settings: &'a Settings,
}

impl<'a> OriginGateway<'a> {
    fn new(settings: &Settings) -> OriginGateway {
        OriginGateway { settings }
    }

    pub fn upsert_origin(
        &self,
        name: String,
        owner_name: String,
        repo_name: String,
        vcs_type: VcsType,
    ) {
        let connection = self.settings.database.get_connection();

        //TODO: this should be wrapped somewhere else, because it is going to be used a lot of places. I think there must be some rustic way of doing this
        let vcs_type = match vcs_type {
            VcsType::Github => String::from("github"),
            _ => String::from("Unknown"),
        };

        let mut statement = connection
            .prepare(
                "INSERT INTO Origins (name, repo_name, owner_name, vcs_type) VALUES(?1,?2,?3,?4) ON CONFLICT(name, vcs_type) DO UPDATE SET repo_name = ?2, owner_name = ?3",
            )
            .unwrap();

        statement
            .execute([&name, &repo_name, &owner_name, &vcs_type])
            .unwrap();
    }
}

pub mod models {
    use crate::configuration::VcsType;

    pub struct OriginRequest {
        pub name: String,
        pub owner_name: String,
        pub repo_name: String,
        pub vcs_type: VcsType,
    }

    pub struct Origin {
        pub name: String,
        pub owner_name: String,
        pub repo_name: String,
        pub vcs_type: VcsType,
    }
}
