use serial_test::serial;
use test_context::test_context;

use application::configuration::{ArgumentParser, Settings, VcsType};
use application::console::Console;
use application::{migrate, Application};
use test_utilities::notification::FakeNotification;
use test_utilities::{
    database::FunctionalTestContext, get_current, get_current_count, get_origin, get_origins_count,
    insert_origin,
};

#[test_context(FunctionalTestContext)]
#[test]
#[serial]
fn shoud_save_only_one_current(context: &FunctionalTestContext) {
    let parser = ArgumentParser {};
    let console = Console {};
    let fake_notification = FakeNotification { callback: || {} };
    let arguments = vec![
        String::from("release_checker"),
        String::from("register"),
        String::from("current"),
        String::from("github"),
        String::from("name"),
        String::from("1"),
        String::from("email@domain.tld"),
    ];

    insert_origin(
        String::from("name"),
        String::from("owner_name"),
        String::from("repo_name"),
        String::from("github"),
        &context.settings.database.get_connection(),
    );

    let application = Application::new(
        &context.settings,
        &console,
        &fake_notification,
        parser,
        arguments,
    );
    application.run();
    application.run();

    let total_count_of_currents = get_current_count(&context.settings.database.get_connection());
    assert_eq!(1, total_count_of_currents);
}

#[test_context(FunctionalTestContext)]
#[test]
#[serial]
fn shoud_update_on_second_insert(context: &FunctionalTestContext) {
    let parser = ArgumentParser {};
    let console = Console {};
    let fake_notification = FakeNotification { callback: || {} };
    let arguments = vec![
        String::from("release_checker"),
        String::from("register"),
        String::from("current"),
        String::from("github"),
        String::from("name"),
        String::from("1"),
        String::from("email@domain.tld"),
    ];

    insert_origin(
        String::from("name"),
        String::from("owner_name"),
        String::from("repo_name"),
        String::from("github"),
        &context.settings.database.get_connection(),
    );

    let application = Application::new(
        &context.settings,
        &console,
        &fake_notification,
        parser,
        arguments,
    );
    application.run();

    let total_count_of_currents = get_origins_count(&context.settings.database.get_connection());
    let current = get_current(
        String::from("name"),
        &context.settings.database.get_connection(),
    );

    assert_eq!(1, total_count_of_currents);
    assert_eq!(current.version, "1");
    assert_eq!(current.name, "name");
    assert_eq!(current.vcs_type, VcsType::Github);

    let parser = ArgumentParser {};
    let settings = Settings::new(None).unwrap();
    let console = Console {};
    let fake_notification = FakeNotification { callback: || {} };

    let arguments = vec![
        String::from("release_checker"),
        String::from("register"),
        String::from("current"),
        String::from("github"),
        String::from("name"),
        String::from("2"),
        String::from("email@domain.tld"),
    ];
    let application = Application::new(&settings, &console, &fake_notification, parser, arguments);
    application.run();

    let total_count_of_currents = get_origins_count(&settings.database.get_connection());
    let current = get_current(String::from("name"), &settings.database.get_connection());

    assert_eq!(1, total_count_of_currents);
    assert_eq!(current.version, "2");
    assert_eq!(current.name, "name");
    assert_eq!(current.vcs_type, VcsType::Github);
}

#[test_context(FunctionalTestContext)]
#[test]
#[serial]
fn should_route_to_current(context: &FunctionalTestContext) {
    let parser = ArgumentParser {};
    let console = Console {};
    let fake_notification = FakeNotification { callback: || {} };

    let arguments = vec![
        String::from("release_checker"),
        String::from("register"),
        String::from("current"),
        String::from("github"),
        String::from("name"),
        String::from("1"),
        String::from("email@domain.tld"),
    ];
    let application = Application::new(
        &context.settings,
        &console,
        &fake_notification,
        parser,
        arguments,
    );
    application.run();
}
